 /**
 * Created by Gifer on 26.05.2016.
 */

 const fs       = require('fs');
 const iconv    = require('iconv-lite');
 const console  = require('gifer-console');

 let Service = {
     formatDate (formatDate, formatString){
        let yyyy = formatDate.getFullYear();
        let yy = yyyy.toString().substring(2);
        let m = formatDate.getMonth() + 1;
        let mm = m < 10 ? "0" + m : m;
        let d = formatDate.getDate();
        let dd = d < 10 ? "0" + d : d;

        let  h = formatDate.getHours();
        let  hh = h < 10 ? "0" + h : h;
        let  n = formatDate.getMinutes();
        let  nn = n < 10 ? "0" + n : n;
        let  s = formatDate.getSeconds();
        let  ss = s < 10 ? "0" + s : s;

        formatString = formatString.replace(/yyyy/i, yyyy);
        formatString = formatString.replace(/yy/i, yy);
        formatString = formatString.replace(/mm/i, mm);
        formatString = formatString.replace(/m/i, m);
        formatString = formatString.replace(/dd/i, dd);
        formatString = formatString.replace(/d/i, d);
        formatString = formatString.replace(/hh/i, hh);
        formatString = formatString.replace(/h/i, h);
        formatString = formatString.replace(/nn/i, nn);
        formatString = formatString.replace(/n/i, n);
        formatString = formatString.replace(/ss/i, ss);
        formatString = formatString.replace(/s/i, s);

        return formatString;
    },

     LoadFile (Path, callback, opt) {
        let _opt = opt || 'win1251';

        fs.stat(Path, function (err, stats) {
            if (err){
                console.err('Failed Load ' + Path + "\n" + err, true);

                if (callback){
                    callback(null);
                }
            } else
            if (stats.size > 20971520){
                console.err('Log file ' + Path + ' too large: ' + stats.size);

                if (callback){
                    callback('FFFF|Log file ' + Path + ' is too large for view, size: ' + stats.size + " bite|||");
                }
            } else {
                let str    = fs.createReadStream(Path);
                let inData = [];

                str.on('data', function(chunk){
                    inData.push(chunk);
                });

                str.on('error', function(err){
                    console.err('Failed Load ' + Path + "\n" + err, true);

                    if (callback){
                        callback(null);
                    }
                });

                str.on('end', function(){
                    try{
                        var res = "";

                        if (opt){
                            res = iconv.decode(Buffer.concat(inData), _opt);
                        } else {
                            res = Buffer.concat(inData).toString();
                        }

                        console.info('Loaded file: "' + Path + '"', console.logLevel.L_Full);
                    } catch (e){
                        console.err('Failed Load ' + Path + "\n" + e.message + "\n" + e.stack + "\n", true);

                        if (callback){
                            callback(null);
                        }
                    }

                    if (callback){
                        callback(res);
                    }
                });
            }
        });
    },

     /**
     * @return {string}
     */
     GetSubstring (data, term){
        let res = "";

        if (data) {
            let idx = data.indexOf(term);

            res = (idx != -1)? data.slice(0, idx) : data.slice(0, data.length + 1);
        }

        return res;
    },

     /**
     * @return {string}
     */
     GetSubstringFrom (str, term){
        let idx = str.indexOf(term);
        let res = (idx != -1)? str.slice(idx + term.length, str.length + 1) : str.slice(0, str.length + 1);

        return (res === '') ? '' : res;
    },

     /**
     * @return {object}
     */
     stringToObj (_data){
        let res = {};

        if (Object.prototype.toString.call( _data ) === '[object Object]'){
            res = _data;
        } else {
            try {
                res = JSON.parse(_data);
            } catch(e){
                console.err("StringToObj: " + e.message);

                res = _data;
            }
        }

        return res;
    },

     /**
      * @return {string}
      */
     dec2hex(i){
         let result = "0000";

         if (i >= 0    && i <= 15)    { result = "000" + i.toString(16); }
         else
         if (i >= 16   && i <= 255)   { result = "00"  + i.toString(16); }
         else
         if (i >= 256  && i <= 4095)  { result = "0"   + i.toString(16); }
         else
         if (i >= 4096 && i <= 65535) { result =         i.toString(16); }

         return result;
    },

     /**
      * @return {string}
      */
     convertToEntities (inData) {
         let result = "";

         if (inData){
             for (let i = 0; i < inData.length; ++i) {
                 if (inData.charCodeAt(i) > 127){
                     result += "\\u" + this.dec2hex(inData.charCodeAt(i));
                 } else {
                     result += inData.charAt(i);
                 }
             }
         } else {
             console.err('convertToEntities: incoming data is empty or undefined', true);
         }

         return result;
    },

     isArray (obj) {
        return Object.prototype.toString.call( obj ) === '[object Array]';
    },

     isObject (obj) {
        return Object.prototype.toString.call( obj ) === '[object Object]';
    },

     isString (obj) {
        return Object.prototype.toString.call( obj ) === '[object String]';
    },

     addLineToFile (text, fileName, remove){
         var fPath = fileName || `${__dirname}/file.out`;

         if (remove){
             try {
                 fs.unlinkSync(fPath);
             } catch (e){}
         }

         fs.appendFileSync(fPath, text);
     },

     ExtractPath(Path) {
         for (var i = Path.length - 1; i >= 0; i--){
             if ((Path[i] === '\\') || (Path[i] === '/')) {
                 return Path.slice(0, i);
             }
         }

         return Path;
     }
 };

 console.DublicateToFile('../logs/' + Service.formatDate(new Date(), 'yyyy/mm/dd') + '/nodewebkit.log');

 module.exports = Service;
